/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
/*
 * clean.c
 */
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

int clean(FILE *, int);

void
main(int argc, char *argv[])
{
	int c, i, err, verbose, xchar;
	FILE *fp;
	char *fname;

	opterr = err = verbose = 0;
	xchar = EOF;

	/*
	 * Process command-line options.
	 */
	while ((c = getopt(argc, argv, "vx:")) != EOF) {
		switch(c) {
		case 'v':
			/*
			 * Use verbose mode.
			 */
			verbose++;
			break;

		case 'x':
			/*
			 * Don't strip the specified character.
			 * Only one exception is allowed.
			 */
			if (xchar != EOF) {
				err++;
				break;
			}
			xchar = atoi(optarg);
			if ((xchar > 255) || (xchar < 0)) {
			  fprintf(stderr, "-x value out of range\n");
			  exit(1);
			}
			break;

		default:
			/*
			 * All other options constitute an error.
			 */
			err++;
		}
	}
	if (err) {
		fprintf(stderr,
		  "usage: clean [-v] [-x char] [files]\n");
		exit(1);
	}

	/*
	 * If no files were specified on the command line,
	 * use stdin.
	 */
	if (optind == argc) {
		c = clean(stdin, xchar);
		if (verbose)
		  fprintf(stderr, "stdin: %d chars cleaned\n", c);
		exit(0);
	}

	/*
	 * Strip each file given on the command line.
	 */
	for (i = optind; i < argc; i++) {
		/*
		 * "-" is a special alias for stdin.
		 */
		if ((argv[i][0] == '-') && (argv[i][1] == '\0')) {
			fp = stdin;
			fname = "stdin";
		} else {
			fname = argv[i];
			fp = fopen(fname, "r");
			if (fp == NULL) {
			  fprintf(stderr, "Cannot open %s: %s\n",
				fname, strerror(errno));
			  err++;
			  continue;
			}
		}
		c = clean(fp, xchar);
		if (verbose)
		  fprintf(stderr, "%s: %d chars cleaned\n",
			fname, c);

		/*
		 * Don't close stdin -- we might need it again.
		 */
		if (fp != stdin)
			fclose(fp);
	}
	exit(err);
}

int
clean(FILE *fp, int xchar)
{
	int c, np;

	np = 0;
	while ((c = getc(fp)) != EOF)
		if (isprint(c) || (c == '\n') || (c == '\t')
		  || (c == xchar))
			putchar(c);
		else
			np++;
	return(np);
}
